"use strict";
// Copyright 2016-2018, Pulumi Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
const azure = require("@pulumi/azure");
const pulumi = require("@pulumi/pulumi");
const util_1 = require("./util");
/**
 * Takes in a callback and a set of bindings, and produces the right AssetMap layout that Azure
 * FunctionApps expect.
 */
function serializeCallback(name, eventSubscriptionArgs, bindingsOutput) {
    if (eventSubscriptionArgs.func && eventSubscriptionArgs.factoryFunc) {
        throw new pulumi.RunError("Cannot provide both [func] and [factoryFunc]");
    }
    if (!eventSubscriptionArgs.func && !eventSubscriptionArgs.factoryFunc) {
        throw new Error("Missing required function callback");
    }
    let func;
    if (eventSubscriptionArgs.func) {
        func = redirectConsoleOutput(eventSubscriptionArgs.func);
    }
    else {
        func = () => {
            const innerFunc = eventSubscriptionArgs.factoryFunc();
            return redirectConsoleOutput(innerFunc);
        };
    }
    const serializedHandlerPromise = pulumi.runtime.serializeFunction(func, { isFactoryFunction: !!eventSubscriptionArgs.factoryFunc });
    const pathSetPromise = pulumi.runtime.computeCodePaths(eventSubscriptionArgs.includePaths || [], eventSubscriptionArgs.includePackages || [], eventSubscriptionArgs.excludePackages || []);
    return pulumi.all([bindingsOutput, serializedHandlerPromise, pathSetPromise]).apply(([bindings, serializedHandler, pathSet]) => {
        const map = {};
        map["host.json"] = new pulumi.asset.StringAsset(JSON.stringify({
            "tracing": {
                "consoleLevel": "verbose",
            },
        }));
        map[`${name}/function.json`] = new pulumi.asset.StringAsset(JSON.stringify({
            "disabled": false,
            "bindings": bindings,
        }));
        map[`${name}/index.js`] = new pulumi.asset.StringAsset(`module.exports = require("./handler").handler`),
            map[`${name}/handler.js`] = new pulumi.asset.StringAsset(serializedHandler.text);
        // TODO: unify this code with aws-serverless instead of straight copying.
        // For each of the required paths, add the corresponding FileArchive or FileAsset to the AssetMap.
        for (const [path, value] of pathSet.entries()) {
            map[name + "/" + path] = value;
        }
        return map;
    });
}
function redirectConsoleOutput(callback) {
    return (context, data) => {
        // Redirect console logging to context logging.
        console.log = context.log;
        console.error = context.log.error;
        console.warn = context.log.warn;
        // tslint:disable-next-line:no-console
        console.info = context.log.info;
        return callback(context, data);
    };
}
/**
 * Base type for all subscription types.
 */
class EventSubscription extends pulumi.ComponentResource {
    constructor(type, name, bindings, args, options) {
        super(type, name, {}, options);
        const parentArgs = { parent: this };
        const appSettings = args.appSettings || pulumi.output({});
        if (!args.resourceGroup) {
            throw new pulumi.RunError("[resourceGroup] must be provided in [args].");
        }
        this.resourceGroup = pulumi.output(args.resourceGroup);
        const resourceGroupName = this.resourceGroup.apply(g => g.name);
        const location = this.resourceGroup.apply(g => g.location);
        const resourceGroupArgs = {
            resourceGroupName,
            location,
        };
        this.storageAccount = args.storageAccount || new azure.storage.Account(`${name}`, Object.assign({}, resourceGroupArgs, { accountKind: "StorageV2", accountTier: "Standard", accountReplicationType: "LRS" }), parentArgs);
        this.storageContainer = args.storageContainer || new azure.storage.Container(`${name}`, {
            resourceGroupName: resourceGroupName,
            storageAccountName: this.storageAccount.name,
            containerAccessType: "private",
        }, parentArgs);
        this.appServicePlan = args.appServicePlan || new azure.appservice.Plan(`${name}`, Object.assign({}, resourceGroupArgs, { kind: "FunctionApp", sku: {
                tier: "Dynamic",
                size: "Y1",
            } }), parentArgs);
        const assetMap = serializeCallback(name, args, bindings);
        // const appSettings = assetAndAppSettings.apply(a => a.appSettings);
        const blob = new azure.storage.ZipBlob(name, {
            resourceGroupName: resourceGroupName,
            storageAccountName: this.storageAccount.name,
            storageContainerName: this.storageContainer.name,
            type: "block",
            content: assetMap.apply(m => new pulumi.asset.AssetArchive(m)),
        }, parentArgs);
        const codeBlobUrl = util_1.signedBlobReadUrl(blob, this.storageAccount, this.storageContainer);
        this.functionApp = new azure.appservice.FunctionApp(name, Object.assign({}, resourceGroupArgs, { appServicePlanId: this.appServicePlan.id, storageConnectionString: this.storageAccount.primaryConnectionString, appSettings: appSettings.apply(settings => (Object.assign({}, settings, { "WEBSITE_RUN_FROM_ZIP": codeBlobUrl }))) }), parentArgs);
    }
}
exports.EventSubscription = EventSubscription;
//# sourceMappingURL=subscription.js.map